@@ -21,6 +21,7 @@
  */
 
 #include "gtksourceencoding.h"
+#include "gtksourceencoding-private.h"
 #include "gtksourceview-i18n.h"
 
 /**
@@ -305,7 +306,8 @@ gtk_source_encoding_lazy_init (void)
  * Gets a #GtkSourceEncoding from a character set such as "UTF-8" or
  * "ISO-8859-1".
  *
- * Returns: the corresponding #GtkSourceEncoding, or %NULL if not found.
+ * Returns: (nullable): the corresponding #GtkSourceEncoding, or %NULL
+ * if not found.
  * Since: 3.14
  */
 const GtkSourceEncoding *
@@ -340,26 +342,28 @@ gtk_source_encoding_get_from_charset (const gchar *charset)
 }
 
 /**
- * gtk_source_encoding_foreach:
- * @func: (scope call): the function to call for each encoding
- * @user_data: user data to pass to the function
+ * gtk_source_encoding_get_all:
  *
- * Calls a function for each encoding.
+ * Gets all encodings.
  *
+ * Returns: (transfer container) (element-type GtkSource.Encoding): a list of
+ * all #GtkSourceEncoding's. Free with g_slist_free().
  * Since: 3.14
  */
-void
-gtk_source_encoding_foreach (GtkSourceEncodingForeachFunc func,
-			     gpointer                     user_data)
+GSList *
+gtk_source_encoding_get_all (void)
 {
+	GSList *all = NULL;
 	gint i;
 
-	func (&utf8_encoding, user_data);
-
-	for (i = 0; i < GTK_SOURCE_ENCODING_LAST; i++)
+	for (i = GTK_SOURCE_ENCODING_LAST - 1; i >= 0; i--)
 	{
-		func (&encodings[i], user_data);
+		all = g_slist_prepend (all, (gpointer) &encodings[i]);
 	}
+
+	all = g_slist_prepend (all, (gpointer) &utf8_encoding);
+
+	return all;
 }
 
 /**
@@ -487,21 +491,140 @@ gtk_source_encoding_get_name (const GtkSourceEncoding* enc)
 	return (enc->name == NULL) ? _("Unknown") : _(enc->name);
 }
 
-/**
- * gtk_source_encoding_get_default_candidates:
+static gboolean
+data_exists (GSList         *list,
+	     const gpointer  data)
+{
+	for (; list != NULL; list = g_slist_next (list))
+	{
+		if (list->data == data)
+		{
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+static GSList *
+strv_to_list (const gchar * const *enc_str)
+{
+	GSList *res = NULL;
+	gchar **p;
+
+	for (p = (gchar **)enc_str; p != NULL && *p != NULL; p++)
+	{
+		const gchar *charset = *p;
+		const GtkSourceEncoding *enc;
+
+		if (g_str_equal (charset, "CURRENT"))
+		{
+			g_get_charset (&charset);
+		}
+
+		g_return_val_if_fail (charset != NULL, NULL);
+		enc = gtk_source_encoding_get_from_charset (charset);
+
+		if (enc != NULL &&
+		    !data_exists (res, (gpointer)enc))
+		{
+			res = g_slist_prepend (res, (gpointer)enc);
+		}
+	}
+
+	return g_slist_reverse (res);
+}
+
+static GSList *
+remove_duplicates_keep_first (GSList *encodings)
+{
+	GSList *new_encodings = NULL;
+	GSList *l;
+
+	for (l = encodings; l != NULL; l = l->next)
+	{
+		gpointer cur_encoding = l->data;
+
+		if (!data_exists (new_encodings, cur_encoding))
+		{
+			new_encodings = g_slist_prepend (new_encodings, cur_encoding);
+		}
+	}
+
+	new_encodings = g_slist_reverse (new_encodings);
+
+	g_slist_free (encodings);
+	return new_encodings;
+}
+
+static GSList *
+remove_duplicates_keep_last (GSList *encodings)
+{
+	GSList *new_encodings = NULL;
+	GSList *l;
+
+	encodings = g_slist_reverse (encodings);
+
+	for (l = encodings; l != NULL; l = l->next)
+	{
+		gpointer cur_encoding = l->data;
+
+		if (!data_exists (new_encodings, cur_encoding))
+		{
+			new_encodings = g_slist_prepend (new_encodings, cur_encoding);
+		}
+	}
+
+	g_slist_free (encodings);
+	return new_encodings;
+}
+
+/*
+ * _gtk_source_encoding_remove_duplicates:
+ * @encodings: (element-type GtkSource.Encoding): a list of #GtkSourceEncoding's.
+ * @removal_type: the #GtkSourceEncodingDuplicates.
+ *
+ * A convenience function to remove duplicated encodings in a list.
+ *
+ * Returns: (transfer container) (element-type GtkSource.Encoding): the new
+ * start of the #GSList.
+ * Since: 3.14
+ */
+GSList *
+_gtk_source_encoding_remove_duplicates (GSList                      *encodings,
+					GtkSourceEncodingDuplicates  removal_type)
+{
+	switch (removal_type)
+	{
+		case GTK_SOURCE_ENCODING_DUPLICATES_KEEP_FIRST:
+			return remove_duplicates_keep_first (encodings);
+
+		case GTK_SOURCE_ENCODING_DUPLICATES_KEEP_LAST:
+			return remove_duplicates_keep_last (encodings);
+
+		default:
+			break;
+	}
+
+	g_return_val_if_reached (encodings);
+}
+
+/*
+ * _gtk_source_encoding_get_default_candidates:
  *
  * Gets the list of default candidate encodings to try when loading a file. See
  * gtk_source_file_loader_set_candidate_encodings().
  *
  * This function returns a different list depending on the current locale (i.e.
- * language, country and default encoding).
+ * language, country and default encoding). The UTF-8 encoding and the current
+ * locale encoding are guaranteed to be present in the returned list.
  *
  * Returns: (transfer container) (element-type GtkSource.Encoding): the list of
  * default candidate encodings. Free with g_slist_free().
  * Since: 3.14
  */
 GSList *
-gtk_source_encoding_get_default_candidates (void)
+_gtk_source_encoding_get_default_candidates (void)
 {
 	const gchar *encodings_str;
 	GVariant *encodings_variant;
@@ -528,7 +651,17 @@ gtk_source_encoding_get_default_candidates (void)
 
 	encodings_strv = g_variant_get_strv (encodings_variant, NULL);
 
-	encodings_list = _gtk_source_encoding_strv_to_list (encodings_strv);
+	encodings_list = strv_to_list (encodings_strv);
+
+	/* Ensure that UTF-8 and CURRENT are present. */
+	encodings_list = g_slist_prepend (encodings_list,
+					  (gpointer) gtk_source_encoding_get_current ());
+
+	encodings_list = g_slist_prepend (encodings_list,
+					  (gpointer) &utf8_encoding);
+
+	encodings_list = _gtk_source_encoding_remove_duplicates (encodings_list,
+								 GTK_SOURCE_ENCODING_DUPLICATES_KEEP_LAST);
 
 	g_variant_unref (encodings_variant);
 	return encodings_list;
@@ -564,73 +697,3 @@ gtk_source_encoding_free (GtkSourceEncoding *enc)
 {
 	g_return_if_fail (enc != NULL);
 }
-
-/* Will probably be used in the future. */
-static gboolean
-data_exists (GSList         *list,
-	     const gpointer  data)
-{
-	for (; list != NULL; list = g_slist_next (list))
-	{
-		if (list->data == data)
-		{
-			return TRUE;
-		}
-	}
-
-	return FALSE;
-}
-
-GSList *
-_gtk_source_encoding_strv_to_list (const gchar * const *enc_str)
-{
-	GSList *res = NULL;
-	gchar **p;
-
-	for (p = (gchar **)enc_str; p != NULL && *p != NULL; p++)
-	{
-		const gchar *charset = *p;
-		const GtkSourceEncoding *enc;
-
-		if (g_str_equal (charset, "CURRENT"))
-		{
-			g_get_charset (&charset);
-		}
-
-		g_return_val_if_fail (charset != NULL, NULL);
-		enc = gtk_source_encoding_get_from_charset (charset);
-
-		if (enc != NULL &&
-		    !data_exists (res, (gpointer)enc))
-		{
-			res = g_slist_prepend (res, (gpointer)enc);
-		}
-	}
-
-	return g_slist_reverse (res);
-}
-
-#if 0
-gchar **
-_gtk_source_encoding_list_to_strv (const GSList *enc_list)
-{
-	GSList *l;
-	GPtrArray *array;
-
-	array = g_ptr_array_sized_new (g_slist_length ((GSList *)enc_list) + 1);
-
-	for (l = (GSList *)enc_list; l != NULL; l = g_slist_next (l))
-	{
-		const GtkSourceEncoding *enc = l->data;
-		const gchar *charset = gtk_source_encoding_get_charset (enc);
-
-		g_return_val_if_fail (charset != NULL, NULL);
-
-		g_ptr_array_add (array, g_strdup (charset));
-	}
-
-	g_ptr_array_add (array, NULL);
-
-	return (gchar **)g_ptr_array_free (array, FALSE);
-}
-#endif
